//------------------------------------------------------------------------------------------------
/// @file EvaluatorGLO.h
/// @author Frolov Andrei
/// @brief    EvaluatorGLO
//------------------------------------------------------------------------------------------------

#ifndef __EVALUATORGLO_H__
#define __EVALUATORGLO_H__

#include <UserModules/UserEvaluator.h>
#include <Mathematics/ContAlignedBox.h>

#include "GeomUtils.h"

#include <vector>
#include <map>
#include <set>
#include <mutex>

using faceMap = std::map<FVAPI_FACE, double>; // -,         ()
using bBox = gte::AlignedBox<3, double>;

static char ERROR_IO_FILE[] = "File with data can't be opened";
static char ERROR_PARAMETER_FILE[] = "File with data has invalid format";


/// @brief ,       ,    
class EvaluatorGLO
{
public:
   EvaluatorGLO() : m_file(nullptr) {}
   ~EvaluatorGLO();

// ,    API
   INTEROP_RESULT SetInitParams(INTEROP_COUNT _numParameters, const FVDLL_PARAMETER* _pParameters);
   INTEROP_DOUBLE EvaluateScalar(FVAPI_CONTEXT _hVarContext);
   INTEROP_RESULT AfterTimeStep();
   INTEROP_RESULT BeforeTimeStep();

protected:
   void ClearCachedData();

   void ReadValues();
   void TransformCoordinates();
   void GetBoundingBox();

   double GetValueInPoint(double* _calcPoint) const;
   double GetPointInterpolatedValue(Vector3& _point) const;
   double GetPointExtrapolatedValue(Vector3& _point) const;
   void FillValBox(sValBox& _box, int x, int y, int z, int iter) const;

//   
   std::vector<Vector3> m_rawCoordinates;
   std::vector<double> m_rawValues;
   std::vector<ValuePoint> m_points;
   bBox m_bBox;
   bBox m_bBoxTolerance;
   PointGrid m_pointGrid;

//   
   FVDLL_PARAMETER_FILE* m_file;
   std::vector<faceMap*> m_HCellFaces;
   std::vector<std::set<FVAPI_FACE>*> m_usedFaces;
   INTEROP_UINT32 m_HCellCount = 0;
   std::mutex m_mutex; //        

//      
   bool m_IsExtrapolationOn = false;
   double m_defaultValue = 0.0;

   Vector3 m_translation;
   double m_angle;
   Vector3 m_rotationVector;

   double m_tolerance = EPS;
   double m_tolerance2 = EPS * EPS;
};

#endif // !__EVALUATORGLO_H__
