#ifndef __UTILSGEOM_H__
#define __UTILSGEOM_H__

#include <Mathematics/Vector3.h>
#include <Mathematics/ContAlignedBox.h>
#include <vector>
#include <set>

constexpr double EPS = 1e-6;
struct ValuePoint;

using Vector3 = gte::Vector3<double>;
using PointContainer = std::vector<ValuePoint*>;

/// @brief     
/// @param _value - .    
/// @param _max  -   
/// @param _min  -   .   .
/// @return     
static inline int GetClippedIndex(double _value, int _max, int _min=0)
{
   return std::max(_min, std::min(static_cast<int>(_value), _max));
}

/// @brief   
static inline double Length2(Vector3 _first)
{
   return gte::Dot(_first, _first);
}

/// @brief  -3       
struct ValuePoint
{
   ValuePoint(Vector3* _coord, double _value)
   {
      coord = _coord;
      m_value = _value;
   }
   Vector3* coord;
   double m_value;
};

/// @brief       
struct Indexes
{
   int x;
   int y;
   int z;
};

/// @brief ,       
struct PointGrid
{
   ~PointGrid();
   void Initialize(std::vector<ValuePoint>& _points, gte::AlignedBox3<double>& _bBox);
   PointContainer* At(size_t x, size_t y, size_t z) const;

   Indexes size;
   Indexes maxInd; // maxInd = size - 1 -     

   int x_shift;    // x_shift = x_size * y_size -       ;
   double splitStep;

   std::vector<PointContainer*> gridBoxes;

};

struct sValBox // ,     
{
   //ValuePoint* x_min;     //direction 0
   //ValuePoint* x_max;     //direction 1
   //ValuePoint* y_min;     //direction 2
   //ValuePoint* y_max;     //direction 3
   //ValuePoint* z_min;     //direction 4
   //ValuePoint* z_max;     //direction 5

   std::vector<double> distances;
   std::vector<ValuePoint*> points;

   Vector3 center;

   sValBox(Vector3& _center);

   bool IsValid() const;
   bool IsEmpty() const;

   void Insert(ValuePoint* _pnt);

   double GetInterpolatedValue() const;
   bool IsInTolerance(double _tolerance) const;
private:
   bool CheckLines(std::set<ValuePoint*>& _points, double _tolerance) const;
   bool CheckPolygons(std::set<ValuePoint*>& _points, double _tolerance) const;
};

#endif // !__UTILSGEOM_H__
