//------------------------------------------------------------------------------------------------
/// @file CoordMainFunctions.cpp
/// @author Frolov Andrei
/// @brief      ,  
//------------------------------------------------------------------------------------------------

#include <UserModules/UserEvaluator.h>
#include "EvaluatorGLO.h"

/// @brief       
std::vector<EvaluatorGLO*> g_instances;

/// @brief  
FVDLL_API fvdllInitModule(FVDLL_INIT_MODULE* _pInitData)
{
   g_instances.reserve(_pInitData->numContexts);
   return INTEROP_OK;
}

/// @brief   
FVDLL_API fvdllInitContext(FVDLL_INIT_CONTEXT* _pInitData, FVDLL_CONTEXT* _outContext)
{
   try
   {
      g_instances.push_back(new EvaluatorGLO());
      *_outContext = g_instances.back();
      g_instances.back()->SetInitParams(_pInitData->numParameters, _pInitData->pParameters);
      return INTEROP_OK;
   }
   catch (INTEROP_RESULT res)
   {
      return res;
   }
   catch (...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

/// @brief    
FVDLL_API fvdllReleaseContexts()
{
   try
   {
      for (size_t i = 0; i < g_instances.size(); ++i)
      {
         delete g_instances[i];
      }
      g_instances.clear();
      return INTEROP_OK;
   }
   catch (...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

/// @brief     
FVDLL_API fvdllBeforeTimeStep(FVDLL_CONTEXT _dllContext)
{
   try
   {
      EvaluatorGLO& evaluator = *reinterpret_cast<EvaluatorGLO*>(_dllContext);
      evaluator.BeforeTimeStep();
      return INTEROP_OK;
   }
   catch (...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

/// @brief     
FVDLL_API fvdllAfterTimeStep(FVDLL_CONTEXT _dllContext)
{
   try
   {
      EvaluatorGLO& evaluator = *reinterpret_cast<EvaluatorGLO*>(_dllContext);
      evaluator.AfterTimeStep();
      return INTEROP_OK;
   }
   catch (...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

/// @brief      
FVDLL_API fvdllEvaluateScalar(FVDLL_CONTEXT _dllContext, FVAPI_CONTEXT _hVarContext, INTEROP_DOUBLE* _outValue)
{
   try
   {
      EvaluatorGLO& evaluator = *reinterpret_cast<EvaluatorGLO*>(_dllContext);
      *_outValue = evaluator.EvaluateScalar(_hVarContext);
      return INTEROP_OK;
   }
   catch (...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

