#include "SinusEvaluatorImpl.h"
#include <UserModules/UserEvaluator.h>
#include <vector>

// Array of all sine evaluator instances
std::vector<CSinusEvaluator*> g_instances;

// Initializing the module
FVDLL_API fvdllInitModule(FVDLL_INIT_MODULE* _pInitData)
{
   g_instances.reserve(_pInitData->numContexts);
   return INTEROP_OK;
}

// Initializing a sine evaluator instance
FVDLL_API fvdllInitContext(FVDLL_INIT_CONTEXT* _pInitData, FVDLL_CONTEXT* _outContext)
{
   try
   {
      g_instances.push_back(new CSinusEvaluator());
      *_outContext = g_instances.back();
      g_instances.back()->SetInitParams(_pInitData->numParameters, _pInitData->pParameters);
      return INTEROP_OK;
   }
   catch(INTEROP_RESULT res)
   {
      return res;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

// Release all instances
FVDLL_API fvdllReleaseContexts()
{
   try
   {
      for (std::vector<CSinusEvaluator*>::iterator ppInstance = g_instances.begin(); ppInstance != g_instances.end(); ++ppInstance)
         delete *ppInstance;

      g_instances.clear();
      return INTEROP_OK;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

// Evaluate scalar value
FVDLL_API fvdllEvaluateScalar(FVDLL_CONTEXT _hContext, FVAPI_CONTEXT _hVarContext, INTEROP_DOUBLE* _outValue)
{
   try
   {
      CSinusEvaluator & ev = *reinterpret_cast<CSinusEvaluator*>(_hContext);

      *_outValue = ev.EvaluateScalar(_hVarContext);

      return INTEROP_OK;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}
