#include "WaveGeneratorBCImpl.h"
#include <UserModules/UserBoundaryCondition.h>
#include <vector>

// Array of all sine evaluator instances
std::vector<CWaveGeneratorBC*> g_instances;

// Initializing the module
FVDLL_API fvdllInitModule(FVDLL_INIT_MODULE* _pInitData)
{
   try
   {
      g_instances.reserve(_pInitData->numContexts);
      return INTEROP_OK;
   }
   catch(INTEROP_RESULT res)
   {
      return res;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

// Initializing a sine evaluator instance
FVDLL_API fvdllInitContext(FVDLL_INIT_CONTEXT* _pInitData, FVDLL_CONTEXT* _outContext)
{
   try
   {
      g_instances.push_back(new CWaveGeneratorBC());
      *_outContext = g_instances.back();
      g_instances.back()->SetInitParams(_pInitData->numParameters, _pInitData->pParameters);
      return INTEROP_OK;
   }
   catch(INTEROP_RESULT res)
   {
      return res;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

// Release all instances
FVDLL_API fvdllReleaseContexts()
{
   try
   {
      for (std::vector<CWaveGeneratorBC*>::iterator ppInstance = g_instances.begin(); ppInstance != g_instances.end(); ++ppInstance)
         delete *ppInstance;

      g_instances.clear();
      return INTEROP_OK;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

FVDLL_API fvdllInitParameterContext(FVDLL_CONTEXT _hContext, FVAPI_VARIABLE _hVariable, INTEROP_STRING _idParam, FVDLL_CONTEXT* _outParameterContext)
{
   try
   {
      CWaveGeneratorBC & instance = *reinterpret_cast<CWaveGeneratorBC*>(_hContext);

     instance.InitParameterContext(_hVariable, _idParam, _outParameterContext);

      return INTEROP_OK;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}

FVDLL_API fvdllGetValueInContext(FVDLL_CONTEXT _hContext, FVDLL_PARAMETER_CONTEXT _hParameterContext, FVAPI_CONTEXT _hVarContext, INTEROP_DOUBLE* _outValue)
{
   try
   {
      CWaveGeneratorBC & instance = *reinterpret_cast<CWaveGeneratorBC*>(_hContext);

      instance.GetValueInContext(_hParameterContext, _hVarContext, _outValue);

      return INTEROP_OK;
   }
   catch(...)
   {
      return INTEROP_ERROR_GENERIC;
   }
}
