#include "WaveGeneratorBCImpl.h"
#include <string>
#include <cmath>
#include <cstring>
#include <memory>

using namespace std;

// Method wrappers
typedef INTEROP_DOUBLE (*GetValueInContext_t)(CWaveGeneratorBC*, FVAPI_CONTEXT, INTEROP_DOUBLE*);

void GetVOFWrapper(CWaveGeneratorBC* _obj, FVAPI_CONTEXT _hVarContext, INTEROP_DOUBLE* _outValue)
{
   *_outValue = _obj->GetVOFInContext(_hVarContext);
}

void GetVelocityFixedWrapper(CWaveGeneratorBC* _obj, FVAPI_CONTEXT _hVarContext, INTEROP_DOUBLE* _outValue)
{
   _obj->GetVelocityFixedInContext(_hVarContext, _outValue);
}

CWaveGeneratorBC::CWaveGeneratorBC()
{
   m_Period = 2.0;
   m_High = 2.0;
   m_Direction[0] = m_Direction[1] = m_Direction[2] = 0.0;
   m_RefPoint[0]  = m_RefPoint[1]  = m_RefPoint[2]  = 0.0;
}

CWaveGeneratorBC::~CWaveGeneratorBC()
{
}

void CWaveGeneratorBC::SetInitParams(INTEROP_COUNT _numParameters, const FVDLL_PARAMETER* _pParameters)
{
   for (INTEROP_COUNT iParam = 0; iParam < _numParameters; ++iParam)
   {
      const FVDLL_PARAMETER & param = _pParameters[iParam];
      if ((strcmp(param.id, "Period") == 0) && (param.type == FVDLL_PARAM_REAL))
      {
         m_Period = param.value.real;
      }
      else if ((strcmp(param.id, "High") == 0) && (param.type == FVDLL_PARAM_REAL))
      {
         m_High = param.value.real;
      }
      else if ((strcmp(param.id, "Direction/X") == 0) && (param.type == FVDLL_PARAM_REAL)) m_Direction[0] = param.value.real;
      else if ((strcmp(param.id, "Direction/Y") == 0) && (param.type == FVDLL_PARAM_REAL)) m_Direction[1] = param.value.real;
      else if ((strcmp(param.id, "Direction/Z") == 0) && (param.type == FVDLL_PARAM_REAL)) m_Direction[2] = param.value.real;
      else if ((strcmp(param.id, "RefPoint/X")  == 0) && (param.type == FVDLL_PARAM_REAL)) m_RefPoint[0]  = param.value.real;
      else if ((strcmp(param.id, "RefPoint/Y")  == 0) && (param.type == FVDLL_PARAM_REAL)) m_RefPoint[1]  = param.value.real;
      else if ((strcmp(param.id, "RefPoint/Z")  == 0) && (param.type == FVDLL_PARAM_REAL)) m_RefPoint[2]  = param.value.real;
      else if ((strcmp(param.id, "ConstVel/X")  == 0) && (param.type == FVDLL_PARAM_REAL)) m_ConstVel[0]  = param.value.real;
      else if ((strcmp(param.id, "ConstVel/Y")  == 0) && (param.type == FVDLL_PARAM_REAL)) m_ConstVel[1]  = param.value.real;
      else if ((strcmp(param.id, "ConstVel/Z")  == 0) && (param.type == FVDLL_PARAM_REAL)) m_ConstVel[2]  = param.value.real;
      else
      {
         throw INTEROP_ERROR_INVALID_PARAMETER;
      }
   }
}

void CWaveGeneratorBC::InitParameterContext(FVAPI_VARIABLE _hVariable, INTEROP_STRING _idParam, FVDLL_PARAMETER_CONTEXT* _outParameterContext)
{
   char varName[100];
   INTEROP_COUNT lenVarName = 100;
   fvapiGetVariableName(_hVariable, &lenVarName, varName);

   if ((strcmp(varName, "VOF") == 0) && (strcmp(_idParam, "WallValue") == 0))
      *_outParameterContext = (FVDLL_PARAMETER_CONTEXT)&GetVOFWrapper;
   else if ((strcmp(varName, "VEL") == 0) && (strcmp(_idParam, "VelVectorFixed") == 0))
      *_outParameterContext = (FVDLL_PARAMETER_CONTEXT)&GetVelocityFixedWrapper;
   else
      throw INTEROP_ERROR_INVALID_PARAMETER;
}

void CWaveGeneratorBC::GetValueInContext(FVDLL_PARAMETER_CONTEXT _hParameterContext, FVAPI_CONTEXT _hVarContext, INTEROP_DOUBLE* _outValue)
{
   GetValueInContext_t ptrGetMethodFunc = (GetValueInContext_t)_hParameterContext;
   (*ptrGetMethodFunc)(this, _hVarContext, _outValue);
}

INTEROP_DOUBLE CWaveGeneratorBC::GetVOFInContext(FVAPI_CONTEXT _hVarContext)
{
   FVAPI_CELL hCell; // cell descriptor
   FVAPI_FACE hFace; // face descriptor
   INTEROP_DOUBLE coordCenter[3]; // face center coordinates
   INTEROP_DOUBLE gravity[3];
   INTEROP_DOUBLE time = 0.0;

   CheckRes(fvapiGetCellOfContext(_hVarContext, &hCell)); // Get cell descriptor
   CheckRes(fvapiGetFaceOfContext(_hVarContext, &hFace)); // Get face descriptor
   CheckRes(fvapiGetFaceCenter(hFace, hCell, coordCenter)); // Get face center coordinates

   CheckRes(fvapiGetCurrentTime(&time)); // Get the current time
   CheckRes(fvapiGetGravityVector(m_RefPoint, gravity)); // Get gravity vector

   INTEROP_DOUBLE PI = 3.14159265358979323846;
   INTEROP_DOUBLE H = m_High;
   INTEROP_DOUBLE g = sqrt(DotProd(gravity, gravity));
   INTEROP_DOUBLE sigma = 2.0 * PI / m_Period;
   INTEROP_DOUBLE k = sigma * sigma / g;

   //       
   coordCenter[0] -= m_ConstVel[0] * time;
   coordCenter[1] -= m_ConstVel[1] * time;
   coordCenter[2] -= m_ConstVel[2] * time;

   //  x       ,       
   INTEROP_DOUBLE planeXCoefD = -1.0 * DotProd(m_Direction, m_RefPoint);
   INTEROP_DOUBLE x = (DotProd(m_Direction, coordCenter) + planeXCoefD) / sqrt(DotProd(m_Direction, m_Direction));

   //  z       ,      
   INTEROP_DOUBLE planeZCoefD = -1.0 * DotProd(gravity, m_RefPoint);
   INTEROP_DOUBLE z = -1.0 * (DotProd(gravity, coordCenter) + planeZCoefD) / sqrt(DotProd(gravity, gravity));

   INTEROP_DOUBLE kxsigmat = k * x - sigma * time;

   INTEROP_DOUBLE eta = H/2*cos(kxsigmat) + H*H*k/16*2*cos(2*kxsigmat);

   if (z <= eta)
   {
      return 1.0;
   }
   else
   {
      return 0.0;
   }
}

void CWaveGeneratorBC::GetVelocityFixedInContext(FVAPI_CONTEXT _hVarContext, INTEROP_DOUBLE* _outValue)
{
   FVAPI_CELL hCell; // cell descriptor
   FVAPI_FACE hFace; // face descriptor
   INTEROP_DOUBLE coordCenter[3]; // face center coordinates
   INTEROP_DOUBLE gravity[3];
   INTEROP_DOUBLE time = 0.0;

   CheckRes(fvapiGetCellOfContext(_hVarContext, &hCell)); // Get cell descriptor
   CheckRes(fvapiGetFaceOfContext(_hVarContext, &hFace)); // Get face descriptor
   CheckRes(fvapiGetFaceCenter(hFace, hCell, coordCenter)); // Get face center coordinates

   CheckRes(fvapiGetCurrentTime(&time)); // Get the current time
   CheckRes(fvapiGetGravityVector(m_RefPoint, gravity)); // Get gravity vector

   INTEROP_DOUBLE PI = 3.14159265358979323846;
   INTEROP_DOUBLE H = m_High;
   INTEROP_DOUBLE g = sqrt(DotProd(gravity, gravity));
   INTEROP_DOUBLE sigma = 2.0 * PI / m_Period;
   INTEROP_DOUBLE k = sigma * sigma / g;

   //       
   coordCenter[0] -= m_ConstVel[0] * time;
   coordCenter[1] -= m_ConstVel[1] * time;
   coordCenter[2] -= m_ConstVel[2] * time;

   //  x       ,       
   INTEROP_DOUBLE planeXCoefD = -1.0 * DotProd(m_Direction, m_RefPoint);
   INTEROP_DOUBLE x = (DotProd(m_Direction, coordCenter) + planeXCoefD) / sqrt(DotProd(m_Direction, m_Direction));

   //  z       ,      
   INTEROP_DOUBLE planeZCoefD = -1.0 * DotProd(gravity, m_RefPoint);
   INTEROP_DOUBLE z = -1.0 * (DotProd(gravity, coordCenter) + planeZCoefD) / sqrt(DotProd(gravity, gravity));

   INTEROP_DOUBLE kxsigmat = k * x - sigma * time;

   INTEROP_DOUBLE eta = H/2*cos(kxsigmat) + H*H*k/16*2*cos(2*kxsigmat);
   INTEROP_DOUBLE u = H/2*g*k/sigma*exp(k*z)*cos(kxsigmat);
   INTEROP_DOUBLE w = H/2*g*k/sigma*exp(k*z)*sin(kxsigmat);

   //   U   
   // U = u * direction / |direction| + w * (-g) / |g|
   INTEROP_DOUBLE U[3];
   INTEROP_DOUBLE lenDirection = DotProd(m_Direction, m_Direction);
   U[0] = u * m_Direction[0] / lenDirection + w * (-gravity[0]) / g;
   U[1] = u * m_Direction[1] / lenDirection + w * (-gravity[1]) / g;
   U[2] = u * m_Direction[2] / lenDirection + w * (-gravity[2]) / g;

   if(z <= eta)
   {
      _outValue[0] = U[0] + m_ConstVel[0];
      _outValue[1] = U[1] + m_ConstVel[1];
      _outValue[2] = U[2] + m_ConstVel[2];
   }
   else
   {
      _outValue[0] = 0.0;
      _outValue[1] = 0.0;
      _outValue[2] = 0.0;
   }
}

INTEROP_DOUBLE CWaveGeneratorBC::DotProd(INTEROP_DOUBLE* _a, INTEROP_DOUBLE* _b)
{
   return _a[0]*_b[0] + _a[1]*_b[1] + _a[2]*_b[2];
}
